// • ▌ ▄ ·.  ▄▄▄·  ▄▄ • ▪   ▄▄· ▄▄▄▄·  ▄▄▄·  ▐▄▄▄  ▄▄▄ .
// ·██ ▐███▪▐█ ▀█ ▐█ ▀ ▪██ ▐█ ▌▪▐█ ▀█▪▐█ ▀█ •█▌ ▐█▐▌·
// ▐█ ▌▐▌▐█·▄█▀▀█ ▄█ ▀█▄▐█·██ ▄▄▐█▀▀█▄▄█▀▀█ ▐█▐ ▐▌▐▀▀▀
// ██ ██▌▐█▌▐█ ▪▐▌▐█▄▪▐█▐█▌▐███▌██▄▪▐█▐█ ▪▐▌██▐ █▌▐█▄▄▌
// ▀▀  █▪▀▀▀ ▀  ▀ ·▀▀▀▀ ▀▀▀·▀▀▀ ·▀▀▀▀  ▀  ▀ ▀▀  █▪ ▀▀▀
//      Magicbane Emulator Project © 2013 - 2022
//                www.magicbane.com


package engine.mobileAI.Threads;
import engine.gameManager.ZoneManager;
import engine.objects.Mob;
import engine.objects.Zone;
import engine.util.ZoneLevel;
import org.pmw.tinylog.Logger;

/**
 * Thread blocks until MagicBane dispatch messages are
 * enqueued then processes them in FIFO order. The collection
 * is thread safe.
 * <p>
 * Any large messages not time sensitive such as load object
 * sent to more than a single individual should be spawned
 * individually on a DispatchMessageThread.
 */

public class MobRespawnThread implements Runnable {

    public MobRespawnThread() {
        Logger.info(" MobRespawnThread thread has started!");

    }

    @Override
    public void run() {

        long startTime = System.currentTimeMillis();
        long rollingKeepFraction = (Zone.rollingAvgMobsAliveDepth - 1) / Zone.rollingAvgMobsAliveDepth;
        long rollingAddFraction = 1 / Zone.rollingAvgMobsAliveDepth;

        while (true) {

            try {

                for (Zone zone : ZoneManager.getAllZones()) {

                    /*
                    if (zone.respawnQue.size() > ZoneLevel.queueLengthToLevelUp) {
                        zone.setCampLvl(zone.getCamplvl() + 1);
                    }
                    else if (zone.respawnQue.isEmpty() &&
                            (zone.lastRespawn + ZoneLevel.msToLevelDown < System.currentTimeMillis()) &&
                            zone.getCamplvl() > 0) {
                        zone.setCampLvl(zone.getCamplvl() - 1);
                    }
                     */

                    int aliveCount = 0;
                    int deadCount = 0;
                    for (Mob mob : zone.zoneMobSet) {
                        if (mob.isAlive()) {
                            aliveCount = aliveCount + 1;
                        }
                        else {
                            deadCount = deadCount + 1;
                        }
                    }

                    zone.rollingAvgMobsAlive =
                            ((zone.rollingAvgMobsAlive * (Zone.rollingAvgMobsAliveDepth - 1) + aliveCount) / Zone.rollingAvgMobsAliveDepth);

                    /*
                    if (startTime + ZoneLevel.msDelayToCampLevel < System.currentTimeMillis()) {
                        if (aliveCount > Math.floor(zone.zoneMobSet.size() / 2.0)) {
                            if (zone.levelUpTimer == 0) {
                                zone.levelUpTimer = System.currentTimeMillis();
                            } else if (zone.levelUpTimer + ZoneLevel.msTolevelUp < System.currentTimeMillis()) {
                                zone.setCampLvl(zone.getCampLvl() + 1);

                                zone.levelUpTimer = 0;
                            }
                        } else if (aliveCount == 0) {

                            if (zone.levelDownTimer == 0) {
                                zone.levelDownTimer = System.currentTimeMillis();
                            } else if (zone.levelDownTimer + ZoneLevel.msToLevelDown < System.currentTimeMillis()) {
                                if (zone.getCampLvl() > 0) {
                                    zone.setCampLvl(zone.getCampLvl() + 1);

                                    zone.levelDownTimer = 0;
                                }
                            }
                        } else {
                            zone.levelUpTimer = 0;
                            zone.levelDownTimer = 0;
                        }
                    }
                    */
                }

                // --------------------------------------------------------------------------------------------------------------------
                // Manage mob respawn
                // --------------------------------------------------------------------------------------------------------------------
                if (!Zone.respawnQue.isEmpty() && Zone.lastRespawn + 100 < System.currentTimeMillis()) {

                    Mob respawner = Zone.respawnQue.iterator().next();

                    if (respawner == null)
                        continue;

                    respawner.respawn();
                    Zone.respawnQue.remove(respawner);
                    Zone.lastRespawn = System.currentTimeMillis();
                }

            } catch (Exception e) {
                Logger.error(e);
            }

        }
    }
    public static void startRespawnThread() {
        Thread respawnThread;
        respawnThread = new Thread(new MobRespawnThread());
        respawnThread.setName("respawnThread");
        respawnThread.start();
    }
}
