package engine.util;

import engine.objects.Zone;

public class ZoneLevel {

    private static final float healthPctPerLevel = (float)0.2;
    private static final float atrPctPerLevel = (float)0.2;
    private static final float defPctPerLevel = (float)0.2;
    private static final float lootPctPerLevel = (float)0.1;
    private static final float goldPctPerLevel = (float)0.2;

    public static final int campLvlAnnounceThreshold = 5;
    public static final int campMaxLvl = 10;

    public static final int queueLengthToLevelUp = 5;
    public static final int msToLevelDown = 60 * 1000;
    public static final int msTolevelUp = 60 * 1000;
    public static final long msDelayToCampLevel = 60 * 1000;

    private static final String[] nameMap =
            {
              "",
              " I",
              " II",
              " III",
              " IV",
              " V",
              " VI",
              " VII",
              " VIII",
              " IX",
              " X"
            };

    public static String getNameSuffix(Zone zone)
    {
        try {
            return nameMap[zone.getCampLvl()];
        }
        catch (Exception ignored)
        {
        }

        return "";
    }

    public static float getMaxHealthPctModifier(Zone zone)
    {
        return getGenericModifier(zone, healthPctPerLevel);
    }

    public static float getAtrPctModifier(Zone zone)
    {
        return getGenericModifier(zone, atrPctPerLevel);
    }

    public static float getDefPctModifier(Zone zone)
    {
        return getGenericModifier(zone, defPctPerLevel);
    }

    public static float getLootDropModifier(Zone zone)
    {
        return getGenericModifier(zone, lootPctPerLevel);
    }

    public static float getGoldDropModifier(Zone zone)
    {
        return getGenericModifier(zone, goldPctPerLevel);
    }

    private static float getGenericModifier(Zone zone, float modifierPerLevel)
    {
        float modifier = (float)1.0;

        if (zone != null)
        {
            modifier += zone.getCampLvl() * modifierPerLevel;
        }

        return modifier;
    }
}
